/*
  (c) 2005 Antonio Cunei, All rights reserved.

  This file is part of the Runabout Emulation Layer for PolyD.

  The Runabout Emulation Layer for PolyD is free software;
  you can redistribute it and/or modify it under the terms of
  the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  The Runabout Emulation Layer for PolyD is distributed
  in the hope that it will be useful, but WITHOUT ANY WARRANTY;
  without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License
  for more details.

  You should have received a copy of the GNU General Public License
  along with the Runabout Emulation Layer for PolyD; if not, write
  to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/

package ovm.polyd.runemu;

import java.util.AbstractCollection;

import ovm.polyd.policy.*;
import ovm.polyd.PolyD;
import ovm.polyd.Factory;

/**
 * A general Runabout emulation, this class can be used as
 * a "drop-in" replacement.<p>Only objects are supported
 * (no primitives). The <code>visitDefault</code> facility is
 * available. If primitives are required, please use
 * {@link RunaboutBis} instead.
 * 
 * @version $Revision: 1.1 $, $Date: 2005/04/02 05:38:36 $
 * @author Antonio Cunei
 *
 */
public class RunaboutCore {

 protected void err(Class c) {
  throw new RunaboutException(
   "The Runabout compatibility layer of PolyD has detected\n"+
   "a visit() call without a proper handler,\n"+
   "when dispatching using "+this.getClass()+",\n"+
   "calling visit("+c.getName()+")"
  );
 }
 protected void visitDefault(Object o)  { err(o.getClass()); }

 /**
  * A "catch-all" <code>visit</code> method, redirected to
  * <code>visitDefault</code>.
  * @param o the object that should be visited
  */
 public void visit(Object o)  { visitDefault(o); }

 /**
  * The bridge method that performs the actual
  * dispatching, calling the appropriate
  * <code>visit</code> for this object.
  * @param obj the object that should be visited
  */
 public void visitAppropriate(Object obj)  { 
  theDispatcher.visit(obj);
 }

 /**
  * The bridge method that performs the dispatching
  * for objects or primitives (not available in this emulation).
  * @param o the object that should be visited
  *  (possibly a wrapped primitive)
  * @param c the class of the object (possibly
  *  a primitive type)
  */
 public void visitAppropriate(Object o,Class c) {
  throw new RunaboutException(
    "RunaboutCore does not handle primitives.\n"+
    "If you need primitives, please use ovm.polyd.runemu.RunaboutBis,\n"+
    "if you only need objects, please use visitAppropriate(Object) instead.");
 }
 
 final private RunaboutBase2 theDispatcher;
 final private static AbstractCollection<Class> registered;
 static {
  registered=new java.util.HashSet<Class>();
 }
 protected RunaboutCore() {
  synchronized (registered) {
   if (!(registered.contains(this.getClass()))) {
    registered.add(this.getClass());
    ovm.polyd.Descriptor<RunaboutBase2> d2;
    d2 = new ovm.polyd.Descriptor<RunaboutBase2>(RunaboutBase2.class,new Class[] {this.getClass()});
    d2.setDispatching(RunaboutDisp.class);
    d2.register();
   }
  }
  theDispatcher = (RunaboutBase2)PolyD.buildFromDescriptor(RunaboutBase2.class, new Object[] {this});
  // System.out.println("Creating new "+this.getClass());
 }
}
