/*
  (c) 2005 Antonio Cunei, All rights reserved.

  This file is part of the Runabout Emulation Layer for PolyD.

  The Runabout Emulation Layer for PolyD is free software;
  you can redistribute it and/or modify it under the terms of
  the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  The Runabout Emulation Layer for PolyD is distributed
  in the hope that it will be useful, but WITHOUT ANY WARRANTY;
  without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License
  for more details.

  You should have received a copy of the GNU General Public License
  along with the Runabout Emulation Layer for PolyD; if not, write
  to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/

package ovm.polyd.runemu;

import java.io.PrintStream;
import java.util.AbstractCollection;
import java.util.HashSet;
import ovm.polyd.Descriptor;
import ovm.polyd.PolyD;
import ovm.polyd.PolyDException;

/**
 * A Runabout emulation that uses PolyD factories to speed
 * up the construction of new dispatchers. In order to
 * take advantage of this emulation, the code of existing
 * programs that use the Runabout neeeds to be slightly
 * altered as follows:<p>
 * 
 * <code>public class Xyz extends RunaboutQuick {<br>
 * &nbsp;private static ovm.polyd.Factory fact=Runabout.prepare(Xyz.class);<br>
 * &nbsp;&nbsp;public Xyz() {<br>
 * &nbsp;&nbsp;super();<br>
 * &nbsp;&nbsp;fact.getDispatcher1(this);<br>
 * &nbsp;}<br>
 * &nbsp;...visit() methods...<br>
 * }</code>
 * 
 * @version $Revision: 1.2 $, $Date: 2005/04/02 07:42:32 $
 * @author Antonio Cunei
 *
 */
public class RunaboutQuick {
 /**
  * Internal use only.
  */
 public RunaboutBase2 the$$$Dispatcher;

 /**
  * Prepares a new <code>Factory</code>, in order to create dispatchers
  * that use <code>RunaboutQuick</code>.
  * @param c the desired subclass of <code>RunaboutQuick</code>
  * @return the new <code>Factory</code>.
  */
 public static ovm.polyd.Factory prepare(Class c) {
  Descriptor d2 = new Descriptor(ovm.polyd.runemu.RunaboutBase2.class, new Class[] {c});
  d2.setDispatching(ovm.polyd.runemu.RunaboutDisp.class);
  java.lang.reflect.Field f=null;
  try {
   f=c.getField("the$$$Dispatcher");
  } catch (Exception e) {
   throw new RunaboutException(
    "Internal error: RunaboutQuick cannot find its own Self filed."
/*#ifdef14
#else14 */
     ,e
//#endif14
   );
  }
  d2.setSelfField(f);
  ovm.polyd.Factory fact=d2.register();
  return fact;
 }
 
 
 /**
  * A "catch-all" <code>visit</code> method. It will
  * not further redispatch the message.
  * @param o the object that should be visited
  */
 public void visit(Object o) { }
 
 /**
  * The bridge method that performs the actual
  * dispatching, calling the appropriate
  * <code>visit</code> for this object.
  * @param obj the object that should be visited
  */
 public void visitAppropriate(Object obj) {
  the$$$Dispatcher.visit(obj);
 }

 protected RunaboutQuick() {}
 
}

