/*
  (c) 2005 Antonio Cunei, All rights reserved.

  This file is part of the Runabout Emulation Layer for PolyD.

  The Runabout Emulation Layer for PolyD is free software;
  you can redistribute it and/or modify it under the terms of
  the GNU General Public License as published by the Free
  Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  The Runabout Emulation Layer for PolyD is distributed
  in the hope that it will be useful, but WITHOUT ANY WARRANTY;
  without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License
  for more details.

  You should have received a copy of the GNU General Public License
  along with the Runabout Emulation Layer for PolyD; if not, write
  to the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/

package ovm.polyd.runemu;
import ovm.polyd.policy.*;

import java.io.PrintStream;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

/**
 * An invocation policy that keeps track of the number of
 * invocations performed. Used by <code>RunaboutStat</code>.
 * 
 * @version $Revision: 1.1 $, $Date: 2005/04/02 05:38:36 $
 * @author Antonio Cunei
 */
public class RunaboutStatInvocation extends PlainInvocation
{
    protected RunaboutStatInvocation() {}
    private static final Invocation theInvocation = new RunaboutStatInvocation();
    /**
     * Returns an instance of this invocation policy.
     * 
     * @return an instance of this policy (a singleton)
     */
    public static Invocation theInvocation()
    {
        return theInvocation;
    }

    private static java.util.Map stats=new java.util.HashMap();

    static private class Counter {
     public int i;
    }
    public Object invoke(Object obj, Method m, Object args[])
        throws Throwable
    {
        Counter ii;
        synchronized (RunaboutStatInvocation.class) {
         if (stats.containsKey(obj.getClass()))
          ii=(Counter)(stats.get(obj.getClass()));
         else {
          ii=new Counter();
          // ii.i=0; // default
          stats.put(obj.getClass(),ii);
         }
         ii.i++;
        }
        Object result = super.invoke(obj, m, args);
        return result;
    }
    
    /**
     * Prints statistics about the invocations performed so far
     * using this invocation policy.
     */
    public static void printStats() {
     synchronized (RunaboutStatInvocation.class) {
      java.util.Iterator ll=stats.keySet().iterator();
      int t=0;
      while (ll.hasNext()) {
       Class c=(Class)(ll.next());
       Counter ii=(Counter)(stats.get(c));
       System.out.println("Invocations: "+c.getName()+" used "+ii.i+" times");
       t+=ii.i;
      }
      System.out.println("Total: "+t+" uses of Runabouts.");
     }
    }

}
